@@ -60,6 +60,8 @@ make_test (" #| #|\n" .
 	   "|##|\n" .
 	   "|#\n" .
 	   "  |# \"\"", "\"\"");
+make_test ("2/4", "1/2");
+make_test ("0.1s2", "10");
 make_test ("'(1 . 2)", "(1 . 2)");
 make_test ("'(1 . (2 3))", "(1 2 3)");
 make_test ("'(1 . 2\n" .
@@ -127,6 +129,7 @@ make_test ("NIL", "()");
 make_test ("t", "T");
 make_test ("(progn)", "()");
 make_test ("(progn 1 2 3)", "3");
+make_test ("(values (+ 1 0) 2 3)", "1\n2\n3");
 make_test ("(if nil 1)", "()");
 make_test ("(common-lisp:if '(1) 2)", "2");
 make_test ("(cl:eq 1 1)", "()");
@@ -137,6 +140,7 @@ make_test ("(cons 1 2)", "(1 . 2)");
 make_test ("(list 1 2 3)", "(1 2 3)");
 make_test ("(append ())", "()");
 make_test ("(append '(1 2) '(3 4))", "(1 2 3 4)");
+make_test ("(append '(1) (values))", "(1)");
 make_test ("(let ((a 2)) a)", "2");
 make_test ("(let ((x 0)) x x x)", "0");
 make_test ("(flet ((a () (write \"\"))) (a))", "\"\"\n\"\"");
@@ -152,8 +156,16 @@ make_test ("(setf b 10)", "10");
 make_test ("(setf b 9)", "9");
 make_test ("(setf c 4)", "4");
 make_test ("c", "4");
+make_test ("(let ((c 0)) c)", "0");
 make_test ("(setf c (+ c 1))", "5");
 make_test ("c", "5");
+make_test ("(setq)", "()");
+make_test ("(setq b 9 c 4 c (+ c 1))", "5");
+make_test ("(setf tt '(a b d))", "(A B D)");
+make_test ("(setf (car tt) 'f)", "F");
+make_test ("tt", "(F B D)");
+make_test ("(setf (cdr tt) 'g)", "G");
+make_test ("tt", "(F . G)");
 make_test ("(let ((b 10)) b)", "10");
 make_test ("(let ((b 1)) (setf b 2) b)", "2");
 make_test ("b", "9");
@@ -187,19 +199,41 @@ make_test ("(g2 4)", "4");
 make_test ("(defun g3 (&optional (x \"a\" provided)) provided)", "G3");
 make_test ("(g3)", "()");
 make_test ("(g3 7)", "T");
-make_test ("(defun fun (x &rest y) y)", "FUN");
-make_test ("(fun 5 6 7 8)", "(6 7 8)");
 make_test ("(defun g4 (x &optional y &rest z) z)", "G4");
 make_test ("(g4 1 2 3 4 5)", "(3 4 5)");
+make_test ("(defun fun (&rest x) x)", "FUN");
+make_test ("(fun)", "()");
+make_test ("(defun fun (x &rest y) y)", "FUN");
+make_test ("(fun 5 6 7 8)", "(6 7 8)");
+make_test ("(defun fun2 (&key a b c) b)", "FUN2");
+make_test ("(fun2 :b 0)", "0");
+make_test ("(fun2 :a 0)", "()");
+make_test ("(fun2)", "()");
+make_test ("(let ((x 0)) (defun inc () (setf x (+ x 1))))", "INC");
+make_test ("(inc)", "1");
+make_test ("(inc)", "2");
+make_test ("(inc)", "3");
 make_test ("(defmacro f () (f))", "F");
 make_test ("(defmacro f () (f))", "F");
 make_test ("(load \"cl.lisp\")", "T");
 make_test ("(1+ 5)", "6");
+make_test ("(1- .5)", "-0.5");
 make_test ("(typep '(1 . 2) 'cons)", "T");
+make_test ("(typep 0 'number)", "T");
+make_test ("(typep 0.1 'real)", "T");
+make_test ("(typep 0 'rational)", "T");
+make_test ("(typep 0.1 'rational)", "()");
 make_test ("(typep 0 'integer)", "T");
 make_test ("(typep 1/2 'ratio)", "T");
+make_test ("(typep 3/1 'ratio)", "()");
+make_test ("(typep 0.1 'float)", "T");
+make_test ("(typep 0.1 'single-float)", "T");
+make_test ("(typep 0.1 'double-float)", "T");
+make_test ("(typep 0.1 'short-float)", "T");
+make_test ("(typep 0.1 'long-float)", "T");
 make_test ("(typep \"abc\" 'string)", "T");
 make_test ("(typep \"abc\" '(string))", "T");
+make_test ("(typep \"abc\" 'array)", "T");
 make_test ("(typep 0 'string)", "()");
 make_test ("(typep '(1 2) 'list)", "T");
 make_test ("(typep #(1 2) 'sequence)", "T");
@@ -208,6 +242,7 @@ make_test ("(typep 0 'nil)", "()");
 make_test ("(typep nil nil)", "()");
 make_test ("(typep nil 'null)", "T");
 make_test ("(typep #p\"\" 'pathname)", "T");
+make_test ("(typep (open #p\"README\") 'stream)", "T");
 make_test ("(make-symbol \"aaa\")", "#:|aaa|");
 make_test ("(boundp 'b)", "T");
 make_test ("(boundp 'g3)", "()");
@@ -218,6 +253,11 @@ make_test ("(fboundp 'g3)", "T");
 make_test ("(symbol-function 'fun)", "#<FUNCTION FUN>");
 make_test ("(symbol-function 'defparameter)", "#<MACRO BUILTIN DEFPARAMETER>");
 make_test ("(symbol-function 'if)", "#<SPECIAL OPERATOR IF>");
+make_test ("(fdefinition 'fun)", "#<FUNCTION FUN>");
+make_test ("(fdefinition 'car)", "#<FUNCTION BUILTIN CAR>");
+make_test ("(special-operator-p 'if)", "T");
+make_test ("(special-operator-p 'car)", "()");
+make_test ("(special-operator-p 'aaa)", "()");
 make_test ("(car '(1 2))", "1");
 make_test ("(cdr '(1 2))", "(2)");
 make_test ("(car ())", "()");
@@ -227,13 +267,31 @@ make_test ("(nth 1 '(0))", "()");
 make_test ("(nthcdr 1 ())", "()");
 make_test ("(nthcdr 2 '(a b c))", "(C)");
 make_test ("(nthcdr 1 '(0 . 1))", "1");
+make_test ("(elt '(0 3) 1)", "3");
+make_test ("(elt \"abc\" 0)", "#\\a");
+make_test ("(elt #(3 2 1) 1)", "2");
+make_test ("(aref #(1 2 3) 1)", "2");
+make_test ("(char \"àbcdef\" 2)", "#\\c");
 make_test ("(length ())", "0");
 make_test ("(length \"aaa\")", "3");
 make_test ("(length #(1 2))", "2");
 make_test ("(length '(0 1 2))", "3");
+make_test ("(list-length '(0 1 2 3))", "4");
+make_test ("(array-dimensions \"aaa\")", "(3)");
+make_test ("(array-dimensions #(1 2 3 4))", "(4)");
+make_test ("(array-dimension \"aaa\" 0)", "3");
 make_test ("(last '(1 2 3))", "(3)");
 make_test ("(last '(1 2 3) 0)", "()");
 make_test ("(last '(1 2 3) 2)", "(2 3)");
+make_test ("(write \"\")", "\"\"\n\"\"");
+make_test ("(write-string \"aaa\\n\")", "aaan\n\"aaan\"");
+make_test ("(write-string \"\n" .
+	   "\")", "\n\"\n\"");
+make_test ("(write-char #\\a)", "a\n#\\a");
+make_test ("(write-char #\\newline)", "\n#\\Newline");
+make_test ("(fresh-line)", "()");
+make_test ("(terpri)", "\n()");
+make_test ("(write-line \"aaa\")", "aaa\n\"aaa\"");
 make_test ("(second '(0))", "()");
 make_test ("(fifth '(0 1 2 3 4))", "4");
 make_test ("(when t \"\" \"\" \"\")", "\"\"");
@@ -243,7 +301,7 @@ make_test ("(tagbody\n" .
 	   "  (go jmp)\n" .
 	   "  (write \"2\")\n" .
 	   "  jmp\n" .
-	   "  (write \"3\"))", "\"1\"\n\"3\"\n()");
+	   "  (write \"3\"))", "\"1\"\"3\"\n()");
 make_test ("(tagbody 1 (go 3) 2 3 (go 4) 4 (write \"\"))", "\"\"\n()");
 make_test ("(tagbody 1 2 (tagbody 3 (go 4)) 4 (write \"\"))", "\"\"\n()");
 make_test ("(cddr '(0 1 2))", "(2)");
@@ -267,6 +325,12 @@ make_test ("(or t nil t)", "T");
 make_test ("(concatenate 'string)", "\"\"");
 make_test ("(concatenate 'string \"\")", "\"\"");
 make_test ("(concatenate 'string \"aa\" \"bb\")", "\"aabb\"");
+make_test ("(dotimes (tmp 10 tmp))", "10");
+make_test ("(mapcar #'car '((1 a) (2 b) (3 c)))", "(1 2 3)");
+make_test ("(mapcar #'abs '(3 -4 2 -5 -6))", "(3 4 2 5 6)");
+make_test ("(mapcar #'cons '(a b c) '(1 2 3))", "((A . 1) (B . 2) (C . 3))");
+make_test ("(cond (t (+ 1 2)))", "3");
+make_test ("(cond (nil 2) (t 3) (nil 4))", "3");
 
 
 # arithmetic tests
@@ -275,11 +339,29 @@ make_test ("(+)", "0");
 make_test ("(+ 1 2 3)", "6");
 make_test ("(+ 1/2 2 3 1000)", "2011/2");
 make_test ("(+ a b)", "17");
-make_test ("(+ .1 .1 .1)", "0.3e0");
-make_test ("(- 3.5)", "-0.35e1");
-make_test ("(- 3 4.5)", "-0.15e1");
+make_test ("(+ .1 .1 .1)", "0.3");
+make_test ("(- 3.5)", "-3.5");
+make_test ("(- 3 4.5)", "-1.5");
 make_test ("(*)", "1");
 make_test ("(* 3 5)", "15");
+make_test ("(/ 12 4)", "3");
+make_test ("(/ 13 4)", "13/4");
+make_test ("(/ -8)", "-1/8");
+make_test ("(/ 3 4 5)", "3/20");
+make_test ("(/ 0.5)", "2");
+make_test ("(/ 60 -2 3 5.0)", "-2");
+make_test ("(floor 1)", "1\n0");
+make_test ("(floor 1/2)", "0\n1/2");
+make_test ("(ceiling 2)", "2\n0");
+make_test ("(ceiling 1/2 1)", "1\n-1/2");
+make_test ("(truncate -3 2)", "-1\n-1");
+make_test ("(truncate -1)", "-1\n0");
+make_test ("(truncate -.1)", "0\n-0.1");
+make_test ("(round .5)", "0\n0.5");
+make_test ("(round -1.5)", "-2\n0.5");
+make_test ("(sqrt 9)", "3");
+make_test ("(sqrt 9.0)", "3");
+make_test ("(isqrt 10)", "3");
 make_test ("(= 1)", "T");
 make_test ("(= 1 1)", "T");
 make_test ("(= 1 2)", "()");
@@ -296,6 +378,15 @@ make_test ("(<= 1 0)", "()");
 make_test ("(> 1.1 0)", "T");
 make_test ("(> 1 1)", "()");
 make_test ("(>= 1 1.0)", "T");
+make_test ("(minusp 0)", "()");
+make_test ("(minusp -0.5)", "T");
+make_test ("(plusp 1)", "T");
+make_test ("(plusp -1)", "()");
+make_test ("(abs 0)", "0");
+make_test ("(abs -0.5)", "0.5");
+make_test ("(abs 1/2)", "1/2");
+make_test ("(zerop -0)", "T");
+make_test ("(zerop 1/2)", "()");
 
 
 print "\ntotal tests: " . $total_tests . "\n";
